@@ -1,12 +1,12 @@
 #' Leverage
 #' 
-#' Computes the leverage of each observation in the PC score / IC mixing matrix (U/M)
-#'  for projection scrubbing. Optionally can threshold the leverage values to 
+#' Computes the leverage of each observation in the PC score (U) or 
+#'  IC mixing (M) matrix for projection scrubbing. Can threshold to 
 #'  flag potential outliers.
 #'
 #' @param Comps The \eqn{n} by \eqn{Q} PC score matrix/IC mixing matrix.
-#' @param are_orthogonal Can the columns of \code{Comps} be assumed to be orthogonal
-#'  and have a 2-norm of 1? Speeds up the computation.
+#' @param are_norm Assume the columns of \code{Comps} are orthogonal
+#'  and have 2-norms equal to 1? Speeds up the computation.
 #' @param median_cutoff The outlier cutoff, in multiples of the median leverage.
 #'  Default: \code{NULL} (do not compute outliers).
 #' 
@@ -18,12 +18,11 @@
 #' @importFrom stats median
 #' 
 #' @export
-leverage <- function(Comps, are_orthogonal=FALSE, median_cutoff=NULL){
-  if (are_orthogonal) {
-    lev <- apply(Comps^2, 1, sum)
+leverage <- function(Comps, are_norm=FALSE, median_cutoff=NULL){
+  lev <- if (are_norm) {
+    colSums(Comps^2)
   } else {
-    #lev <- diag( Comps %*% solve(t(Comps) %*% Comps, t(Comps)) )
-    lev <- diag(hat_matrix(Comps))
+    diag(hat_matrix(Comps))
   }
 
   out <- list(meas=lev)
