@@ -1,13 +1,13 @@
 test_that("pscrub works", {
   psx <- testthat::expect_warning(fMRIscrub:::pscrub_multi(
     Dat1,
-    projection = "all"
+    projection = c("ICA", "ICA_kurt"), # "all"
   ))
   myplot <- fMRIscrub:::plot.scrub_projection_multi(psx)
 
   psx <- testthat::expect_warning(fMRIscrub:::pscrub_multi(
     Dat2,
-    projection = c("fusedPCA", "ICA_kurt"),
+    projection = c("PCA", "ICA_kurt"),
     kurt_quantile = .90,
     cutoff = 5,
     verbose = TRUE
@@ -17,11 +17,11 @@ test_that("pscrub works", {
   psx <- testthat::expect_warning(pscrub(Dat1))
   plot(psx)
 
-  psx <- pscrub(
-    matrix(rnorm(10000), nrow=50), "fusedPCA", 1, center=FALSE, PESEL=FALSE, kurt_quantile=.8,
-    full_PCA = TRUE, cutoff=5, verbose=TRUE
-  )
-  plot(psx)
+  # psx <- pscrub(
+  #   matrix(rnorm(4000), nrow=30), "fusedPCA", 1, center=FALSE, PESEL=FALSE, kurt_quantile=.8,
+  #   full_PCA = TRUE, cutoff=5, verbose=TRUE
+  # )
+  # plot(psx)
 
   psx <- testthat::expect_warning(pscrub(
     matrix(rnorm(10000), ncol=50)
